// LICENSE : MIT
"use strict";

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _AllLiteral = require("./tag/AllLiteral");

var _FunctionType = require("./tag/FunctionType");

var _NameExpression = require("./tag/NameExpression");

var _NonNullableType = require("./tag/NonNullableType");

var _NullableType = require("./tag/NullableType");

var _RecordType = require("./tag/RecordType");

var _RestType = require("./tag/RestType");

var _TypeApplication = require("./tag/TypeApplication");

var _UnionType = require("./tag/UnionType");

var _CodeGenerator = require("./CodeGenerator");

var _CodeGenerator2 = _interopRequireDefault(_CodeGenerator);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var ObjectAssign = require("object-assign");

/**
 * @typedef {Object} AssertGeneratorOptions
 * @property {Function} generator
 */
var defaultOptions = {
    Generator: _CodeGenerator2.default
};

var AssertGenerator = function () {
    function AssertGenerator() {
        _classCallCheck(this, AssertGenerator);
    }

    _createClass(AssertGenerator, null, [{
        key: "createAsserts",

        /**
         *
         * @param {Array<Object>} comments AST's comment nodes. it should be BlockComment
         * @param {AssertGeneratorOptions} options
         * @returns {Array<String>} array of assertion
         */
        value: function createAsserts(comments) {
            var options = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

            if (comments == null) {
                return [];
            }
            var Generator = options.Generator || defaultOptions.Generator;
            var isNotEmpty = function isNotEmpty(tag) {
                return tag != null;
            };
            // primitive
            var createTag = function createTag(tag) {
                return AssertGenerator.createAssertFromTag(tag, Generator);
            };
            return comments.tags.map(createTag).filter(isNotEmpty);
        }

        /**
         * create assertions from @type
         * @param {string} name name is variable name
         * @param {Array<Object>} comments AST's comment nodes. it should be BlockComment
         * @param {AssertGeneratorOptions} options
         * @returns {Array<String>} array of assertion
         */

    }, {
        key: "createTypeAsserts",
        value: function createTypeAsserts(name, comments) {
            var options = arguments.length <= 2 || arguments[2] === undefined ? {} : arguments[2];

            if (comments == null) {
                return [];
            }
            var Generator = options.Generator || defaultOptions.Generator;
            var isNotEmpty = function isNotEmpty(tag) {
                return tag != null;
            };
            // primitive
            var createTag = function createTag(tag) {
                return AssertGenerator.createAssertFromTypeTag(name, tag, Generator);
            };
            return comments.tags.map(createTag).filter(isNotEmpty);
        }

        /**
         * create assertion string from `typeNode` and `name`.
         * @param {string} [name] variable name
         * @param {{title:string, description: ?string, type:Object}} typeNode
         * @param {CodeGenerator} Generator
         * @returns {string|undefined}
         */

    }, {
        key: "createAssertFromTypeTag",
        value: function createAssertFromTypeTag(name, typeNode) {
            var Generator = arguments.length <= 2 || arguments[2] === undefined ? _CodeGenerator2.default : arguments[2];

            if (name === undefined) {
                return;
            }
            var title = typeNode.title;
            // @returns etc... are not param
            if (title !== "type") {
                return;
            }
            // @param x - x have not type
            if (typeNode.type == null) {
                return;
            }

            var node = ObjectAssign({}, {
                name: name
            }, typeNode);
            var generator = new Generator(node);
            var tagType = typeNode.type.type;
            if (tagType === "NameExpression") {
                return (0, _NameExpression.NameExpression)(node, generator);
            } else if (tagType === "AllLiteral") {
                return (0, _AllLiteral.AllLiteral)(node, generator);
            } else if (tagType === "FunctionType") {
                return (0, _FunctionType.FunctionType)(node, generator);
            } else if (tagType === "RecordType") {
                return (0, _RecordType.RecordType)(node, generator);
            } else if (tagType === "UnionType") {
                return (0, _UnionType.UnionType)(node, generator);
            } else if (tagType === "TypeApplication") {
                return (0, _TypeApplication.TypeApplication)(node, generator);
            } else if (tagType === "RestType") {
                return (0, _RestType.RestType)(node, generator);
            } else if (tagType === "NullableType") {
                return (0, _NullableType.NullableType)(node, generator);
            } else if (tagType === "NonNullableType") {
                return (0, _NonNullableType.NonNullableType)(node, generator);
            }
        }

        /**
         * @param tagNode tagNode is defined by doctorin
         * @param {CodeGenerator} Generator
         * @return {string|undefined} return assertion code string
         * Reference https://esdoc.org/tags.html#type-syntax
         * https://github.com/eslint/doctrine/blob/master/test/parse.js
         */

    }, {
        key: "createAssertFromTag",
        value: function createAssertFromTag(tagNode) {
            var Generator = arguments.length <= 1 || arguments[1] === undefined ? _CodeGenerator2.default : arguments[1];

            var title = tagNode.title;
            // @returns etc... are not param
            if (title !== "param") {
                return;
            }
            // @param x - x have not type
            if (tagNode.type == null) {
                return;
            }
            var generator = new Generator(tagNode);
            var tagType = tagNode.type.type;
            if (tagType === "NameExpression") {
                return (0, _NameExpression.NameExpression)(tagNode, generator);
            } else if (tagType === "AllLiteral") {
                return (0, _AllLiteral.AllLiteral)(tagNode, generator);
            } else if (tagType === "FunctionType") {
                return (0, _FunctionType.FunctionType)(tagNode, generator);
            } else if (tagType === "RecordType") {
                return (0, _RecordType.RecordType)(tagNode, generator);
            } else if (tagType === "UnionType") {
                return (0, _UnionType.UnionType)(tagNode, generator);
            } else if (tagType === "TypeApplication") {
                return (0, _TypeApplication.TypeApplication)(tagNode, generator);
            } else if (tagType === "RestType") {
                return (0, _RestType.RestType)(tagNode, generator);
            } else if (tagType === "NullableType") {
                return (0, _NullableType.NullableType)(tagNode, generator);
            } else if (tagType === "NonNullableType") {
                return (0, _NonNullableType.NonNullableType)(tagNode, generator);
            }
        }
    }]);

    return AssertGenerator;
}();

exports.default = AssertGenerator;
;
//# sourceMappingURL=AssertGenerator.js.map